package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamDetailCourseDto;
import com.qmth.distributed.print.business.bean.dto.SyncExamTaskDto;
import com.qmth.distributed.print.business.entity.ExamDetailCourse;
import com.qmth.distributed.print.business.enums.CardTypeEnum;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.enums.SyncCardTypeEnum;
import com.qmth.distributed.print.business.mapper.ExamDetailCourseMapper;
import com.qmth.distributed.print.business.service.ExamCardService;
import com.qmth.distributed.print.business.service.ExamDetailCourseService;
import com.qmth.distributed.print.business.service.ExamTaskDetailService;
import com.qmth.teachcloud.common.bean.result.TbTaskDetailResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.CardCreateMethodEnum;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.SysOrgService;
import com.qmth.teachcloud.common.service.TeachcloudCommonService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考务数据（考场关联科目） 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamDetailCourseServiceImpl extends ServiceImpl<ExamDetailCourseMapper, ExamDetailCourse> implements ExamDetailCourseService {

    @Resource
    private ExamTaskDetailService examTaskDetailService;

    @Resource
    private ExamCardService examCardService;

    @Resource
    private SysOrgService sysOrgService;

    @Resource
    private TeachcloudCommonService teachcloudCommonService;

    @Override
    public List<Map<String, Object>> listClientByExamDetailIdAndStatus(Long examDetailId) {
        return this.baseMapper.listClientByExamDetailIdAndStatus(examDetailId, ExamDetailStatusEnum.FINISH.name());
    }

    @Override
    public void updatePaperNumber(List<Long> examDetailCourseIds, String paperNumber, String relatePaperType) {
        UpdateWrapper<ExamDetailCourse> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetailCourse::getPaperNumber, paperNumber)
                .set(ExamDetailCourse::getPaperType, relatePaperType)
                .in(ExamDetailCourse::getId, examDetailCourseIds);
        this.update(updateWrapper);
    }

    @Override
    public List<ExamDetailCourse> listByExamDetailIdAndStatus(Long examDetailId) {
        return this.baseMapper.listByExamDetailIdAndStatus(examDetailId, ExamStatusEnum.SUBMIT.name());
    }

    @Override
    public List<BasicCourse> listCoursesByPrintPlanId(String param, List<Long> printPlanIdList) {
        return this.baseMapper.listCoursesByPrintPlanId(SystemConstant.translateSpecificSign(param), printPlanIdList);
    }

    @Override
    public List<ExamDetailCourse> listByPrintPlanIdAndCourseCodeAndPaperNumber(Long schoolId, Long printPlanId, String courseCode, String paperNumber) {
        return this.baseMapper.listByPrintPlanIdAndCourseCodeAndPaperNumber(schoolId, printPlanId, courseCode, paperNumber);
    }

    @Override
    public List<ExamDetailCourse> listByCourseCodeAndPaperNumber(Long schoolId, Long examId, String courseCode, String paperNumber) {
        return this.baseMapper.listByAndCourseCodeAndPaperNumber(schoolId, examId, courseCode, paperNumber);
    }

    @Override
    public List<ExamDetailCourseDto> listByPrintPlanIdAndExamTaskId(List<Long> printPlanIds, Long examTaskId, String paperType) {
        return this.baseMapper.listByPrintPlanIdAndExamTaskId(printPlanIds, examTaskId, paperType);
    }

    @Override
    public List<ExamDetailCourse> listExamDetailByExamIdAndPaperNumber(Long schoolId, Long examId, String paperNumber) {
        return this.baseMapper.listExamDetailByExamIdAndPaperNumber(schoolId, examId, paperNumber);
    }

    @Override
    public TbTaskDetailResult getByExamDetailId(Long examDetailId) {
        return this.baseMapper.getByExamDetailId(examDetailId);
    }

    @Override
    public List<String> listPaperNumberByPrintPlanId(String param, List<Long> printPlanIdList, Set<Long> orgIds) {
        return this.baseMapper.listPaperNumberByPrintPlanId(SystemConstant.translateSpecificSign(param), printPlanIdList, orgIds);
    }

    @Override
    public List<ExamDetailCourse> listByExamDetailId(Long examDetailId) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<ExamDetailCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamDetailCourse::getSchoolId, schoolId)
                .eq(ExamDetailCourse::getExamDetailId, examDetailId);
        return this.list(queryWrapper);
    }
}
